%% parameter settings for testing
load ('partLocConfig.mat');
all_I_fold='./image';             % image fold
all_I_n=dir(all_I_fold); 
all_W_fold='./test_working';      % working fold
mkdir(all_W_fold);                
all_S_fold='./score';             % score fold
mkdir(all_S_fold);
% note parameters in training process can be different from testing process
resolutionGap=0.1;
numExtend = 1;                               % number of gaps extended both below and above zero
numResolution = numExtend*2 + 1;             % number of resolutions to search for in detection stage
originalResolution = numExtend + 1;          % original resolution is the one at which the imresize factor = 1
allResolution = (-numExtend : numExtend)*resolutionGap + 1.;
rotationRange = -1:1:1;                      % allowed global rotation of the template
%% read images
for all_I_c=3:length(all_I_n)
    %%
    mkdir(['test_working/' all_I_n(all_I_c).name]);          % working directory to store .mat files. 
    imageFolder = [all_I_fold '/' all_I_n(all_I_c).name];    % folder of training images  
    imageName = dir([imageFolder '/' img_format]);
    numImage = length(testID(all_I_c-2).index);
    Ioriginal = cell(1, numImage);
    for img = 1 : numImage
        tmpIm = imread([imageFolder '/' imageName(testID(all_I_c-2).index(img)).name]); 
        if size(tmpIm,3) == 3
            tmpIm = rgb2gray(tmpIm);
        end
        sx = size(tmpIm,1); sy = size(tmpIm,2);
        if resizeTrainingImages
            tmpIm = imresize( tmpIm, constantImageArea/sqrt(sx*sy), 'bilinear' );
        end
        Ioriginal{img} = single(tmpIm);
        J0 = Ioriginal{img};
        J = cell(1, numResolution);
        C = cell(1, numResolution);
        posX=cell(1, numResolution);
        posY=cell(1, numResolution);
        for r=1:numResolution
           J{r} = imresize(J0, allResolution(r), 'nearest');  % scaled images
        end
        multipleResolutionImageName = ['test_working/' all_I_n(all_I_c).name '/' 'multipleResolutionImage' num2str(img)]; 
        save(multipleResolutionImageName, 'J');
    end
    clear Ioriginal;
end;
clear anno;

count = 0;
% generate the set of geometric transformations for each template
for templateScaleInd = 0:0 % large scale change
    for rotation = rotationRange 
        for rowScale = 2.^[0] % small scale change and aspect ratio change
            for colScale = 2.^[0]
                count = count + 1;
                templateTransform{count} = [templateScaleInd rowScale colScale rotation];
            end
        end
    end
end
nTransform = count;
% prepare morph-back mappings for geometric transforms
largestPartSizeX = templateSize(1); largestPartSizeY = templateSize(2);
denseX = -floor(largestPartSizeX/2) + (1:largestPartSizeX);
denseY = -floor(largestPartSizeY/2) + (1:largestPartSizeY);
count = 0;
inRow = zeros(length(denseX)*length(denseY),1,'single');
inCol = zeros(length(denseX)*length(denseY),1,'single');
inO = zeros(numel(inRow),1,'single'); 
inS = zeros(numel(inRow),1,'single');
for y = denseY
    for x = denseX
        count = count+1;
        inRow(count) = x;
        inCol(count) = y;
    end
end
outRow = cell(nTransform,1);
outCol = cell(nTransform,1);
for iT = 1:nTransform
	tScale = templateTransform{iT}(1); 
	rScale = templateTransform{iT}(2);
	cScale = templateTransform{iT}(3);
	[outRow{iT}, outCol{iT}] = ...
		mexc_TemplateAffineTransform(tScale,rScale,cScale,...
		    templateTransform{iT}(4),inRow,inCol,inO,inS,numOrient);
end

save('PamaTest', 'numExtend', 'numResolution', 'originalResolution','allResolution', 'all_I_fold', 'all_W_fold', 'all_S_fold','all_I_n','rotationRange','nTransform','templateTransform');