disp(['===============> Getting parameters, mex-c code, and training images']);
mkdir('working'); % working directory to store .mat files. 
mkdir('output');  % directory that contains output from the code. 
%% parameters for the whole system
trainsize = 15;               % number of training images per category
testsize = 15;                % number of testing images per category
img_fold = './image';         % image fold that includes both training and testing images
img_format = '*.jpg';         % image format for reading
RandomSplit;                  % split images as training and testing
save trainsize trainsize;
save testsize testsize;
save img_format img_format;
%% parameters for the outer square bounding box
category = '1';
supressionModeInEStep = 'MatchingPursuit'; % matching pursuit in E step (other options: LocalSurroundSurpression)
templateSize = [64 64];
templateSize = single(templateSize);
partSize = floor(sqrt(templateSize(1)*templateSize(2))); % alias for the template size (radius)
resizeTrainingImages = true;
constantImageArea = 150^2; % for resizing images (keep a constant area)
numRandomStart = 1;
emptyThre = 20;
%% parameters for EM clustering
locationPerturbationFraction = .4; % the size of neighborhood for MAX2 pooling, as well as surround supression
locationPerturbationFraction_final = .4; % (used in later EM iterations) the size of neighborhood for MAX2 pooling, as well as surround supression
rotationRange = -2:2:2; % allowed global rotation of the template
subsampleS2 = 1; % subsampling step size for computing SUM2 maps
S2Thres = 20;     % cut-off value for detected instances of the template
S1softthres = 0.0; % soft thresholding cutting-off for S1 maps
numIter = 10; % number of EM iterations
%% parameters for active basis
numCluster = 200; % number of data clusters
maxNumClusterMember = 100; % (no need to change) maximum number of training examples in each cluster used in re-learning the template
numElement = S2Thres; % number of Gabors in active basis at the first scale
epsilon = .1; % allowed correlation between selected Gabors 
subsample = 1; subsampleM1 = 1; % subsample in computing MAX1 maps
locationShiftLimit = 3; % shift in normal direction = locationShiftLimit*subsample pixels
orientShiftLimit = 1; % shift in orientation
%% parameters for detection (controls scaling of templates)
inhibitFind = -1;  % whether to apply inhibition after detection for re-computing MAX2 score
resolutionGap = .2; % gap between consecutive resolutions in detection
numExtend=1;                      % number of gaps extended both below and above zero
numResolution = numExtend*2 + 1;  % number of resolutions to search for in detection stage
originalResolution = numExtend + 1; % original resolution is the one at which the imresize factor = 1
allResolution = (-numExtend : numExtend)*resolutionGap + 1.;
%% parameters for Gabor filters
numScale = 1; % number of Gabor scales (don't change it)
scales = 0.7; % scales of Gabor wavelets 
numOrient = 16;  % number of orientations
saturation = 6.; % saturation level for sigmoid transformation
doubleOrNot = -1;
%% parameters for exponential model
binSize = .2;  % binsize for computing histogram of q()
numStoredPoint = 50; % number of stored lambda values
spacing = .1; % spacing between two adjacent points
%% parameters for normalization
localOrNot = 1; % if we use local normalization or not. If not, set it to -1 
localHalfx = 20; localHalfy = 20; % the half range for local normalization, has to be quite large
localHalfx1 = 20; localHalfy1 = 20;
windowNormalizeOrNot = -1; % whether normalize within the scanning window in detection 
if (localOrNot>0)
    windowNormalizeOrNot = -1;
end % if we use local normalization, we should not use window normalization in detection
thresholdFactor = .01;  % divide the response by max(average, maxAverage*thresholdFactor)
%% read training images
sizeTemplatex = templateSize(1);
sizeTemplatey = templateSize(2);
halfTemplatex = floor(sizeTemplatex/2);
halfTemplatey = floor(sizeTemplatey/2);
numImage=length(trainID)*trainsize;
Ioriginal = cell(1, numImage);
Joriginal = cell(numResolution,numImage);
for i=3:length(Con)
    sub_img_fold=[img_fold '/' Con(i).name];
    sub_img=dir([sub_img_fold '/' img_format]);
    imgStart = (i-3)*trainsize+1;
    imgStop = (i-3)*trainsize+trainsize;
    for img=imgStart:imgStop
        j = img-(i-3)*trainsize;
        tmpIm=imread([sub_img_fold '/' sub_img(trainID(i-2).index(j)).name]);
        if size(tmpIm,3) == 3
            tmpIm = rgb2gray(tmpIm);
        end
        sx = size(tmpIm,1); sy = size(tmpIm,2);
        if resizeTrainingImages
            tmpIm = imresize( tmpIm, sqrt(constantImageArea/(sx*sy)), 'bilinear' );
        end
        Ioriginal{img} = single(tmpIm);
        J0 = single(tmpIm);
        for r=1:numResolution
            Joriginal{r,img} = imresize(J0, allResolution(r), 'nearest');  % scaled images
        end
        %multipleResolutionImageName = ['working/multipleResolutionImage' num2str(img)];
        %save(multipleResolutionImageName, 'J');
    end;
end;
        multipleResolutionImageName = ['working/multipleResolutionImages.mat'];
        save(multipleResolutionImageName, 'Joriginal','Ioriginal');
count = 0;
% generate the set of geometric transformations for each template
for templateScaleInd = 0:0 % large scale change
    for rotation = rotationRange 
        for rowScale = 2.^[0] % small scale change and aspect ratio change
            for colScale = 2.^[0]
                count = count + 1;
                templateTransform{count} = [templateScaleInd rowScale colScale rotation];
            end
        end
    end
end
nTransform = count;
% prepare morph-back mappings for geometric transforms
largestPartSizeX = templateSize(1); largestPartSizeY = templateSize(2);
denseX = -floor(largestPartSizeX/2) + (1:largestPartSizeX);
denseY = -floor(largestPartSizeY/2) + (1:largestPartSizeY);
count = 0;
inRow = zeros(length(denseX)*length(denseY),1,'single');
inCol = zeros(length(denseX)*length(denseY),1,'single');
inO = zeros(numel(inRow),1,'single'); 
inS = zeros(numel(inRow),1,'single');
for y = denseY
    for x = denseX
        count = count+1;
        inRow(count) = x;
        inCol(count) = y;
    end
end
outRow = cell(nTransform,1);
outCol = cell(nTransform,1);
for iT = 1:nTransform
	tScale = templateTransform{iT}(1); 
	rScale = templateTransform{iT}(2);
	cScale = templateTransform{iT}(3);
	[outRow{iT}, outCol{iT}] = ...
		mexc_TemplateAffineTransform(tScale,rScale,cScale,...
		    templateTransform{iT}(4),inRow,inCol,inO,inS,numOrient);
end
save('partLocConfig.mat','templateSize','scales','numRandomStart','maxNumClusterMember',...
	'S2Thres','S1softthres','allResolution','resizeTrainingImages','constantImageArea',...
    'category','numOrient','localOrNot','subsample','saturation','locationShiftLimit','orientShiftLimit',...
    'numElement','thresholdFactor','doubleOrNot', 'numCluster', 'numIter', 'subsampleS2', 'locationPerturbationFraction','numScale',...
    'partSize','rotationRange','nTransform','templateTransform','S1softthres','subsampleM1','localHalfx','localHalfy',...
    'inRow', 'inCol', 'outRow', 'outCol', 'largestPartSizeX', 'largestPartSizeY','locationPerturbationFraction_final');


%% parameter settings for testing
all_I_fold='./image';             % image fold
all_I_n=dir(all_I_fold); 
all_W_fold='./test_working';      % working fold
mkdir(all_W_fold);                
all_S_fold='./score';             % score fold
mkdir(all_S_fold);
numExtendTest = 1; % number of gaps extended both below and above zero
numResolutionTest = numExtendTest*2 + 1;  % number of resolutions to search for in detection stage
originalResolutionTest = numExtendTest + 1; % original resolution is the one at which the imresize factor = 1
allResolutionTest = (-numExtendTest : numExtendTest)*resolutionGap + 1.;
save('PamaTest', 'numExtendTest', 'numResolutionTest', 'originalResolutionTest','allResolutionTest', 'all_I_fold', 'all_W_fold', 'all_S_fold','all_I_n');
%% read images
for all_I_c=3:length(all_I_n)
    %%
    mkdir(['test_working/' all_I_n(all_I_c).name]);        % working directory to store .mat files. 
    imageFolder = [all_I_fold '/' all_I_n(all_I_c).name];  % folder of training images  
    imageName = dir([imageFolder '/' img_format]);
    numTestImage = length(testID(all_I_c-2).index);
    IoriginalTest = cell(1, numTestImage);
    JoriginalTest = cell(numResolution,numTestImage);
    for img = 1 : numTestImage
        tmpIm = imread([imageFolder '/' imageName(testID(all_I_c-2).index(img)).name]); 
        if size(tmpIm,3) == 3
            tmpIm = rgb2gray(tmpIm);
        end
        sx = size(tmpIm,1); sy = size(tmpIm,2);
        if resizeTrainingImages
            tmpIm = imresize( tmpIm, sqrt(constantImageArea/(sx*sy)), 'bilinear' );
        end
        IoriginalTest{img} = single(tmpIm);
        J0 = IoriginalTest{img};
        for r=1:numResolutionTest
           JoriginalTest{r,img} = imresize(J0, allResolutionTest(r), 'nearest');  % scaled images
        end
    end
    multipleResolutionImageName = ['test_working/' all_I_n(all_I_c).name '/' 'multipleResolutionImages.mat']; 
    save(multipleResolutionImageName, 'JoriginalTest','IoriginalTest');

    clear IoriginalTest;
end;
